#ifndef CAMERA_H
#define CAMERA_H

#include <QGLViewer/frame.h>
#include "ray.h"

class Scene;

class Camera
{
public :
  Camera();
  
  qglviewer::Frame& frame() { return frame_; };  
  const qglviewer::Frame& frame() const { return frame_; };
  float fieldOfView() const { return fieldOfView_; };
  int xResolution() const { return xResolution_; };
  int yResolution() const { return yResolution_; };
  
  void setFrame(const qglviewer::Frame& frame) { frame_ = frame; };
  // fieldOfView is the VERTICAL value (in radians).
  // Horizontal field of view is infered from xResolution/yResolution ratio.
  void setFieldOfView(float fieldOfView) { fieldOfView_ = fieldOfView; compGeometry(); };
  void setXResolution(int xResolution);
  void setYResolution(int yResolution);
  
  void initFromDOMElement(const QDomElement& e);
  void draw(float radius) const;

  void drawCamera(float halfHeight, float halfWidth, float dist) const;

  Ray getPixelRay(float x, float y) const;
  void drawAllRays() const;
  void testAllRays() const;
  
private:  
  void compGeometry();

  qglviewer::Frame frame_;
  float fieldOfView_;
  int xResolution_, yResolution_;

  float dist_;
  float halfHeight_;
  float halfWidth_;
};


// Compute halfHeight_ and halfWidth_ each time the fieldOfView, the xResolution or the yResolution is changed
inline
void Camera::compGeometry() {
	float aspect=(float)xResolution()/(float)yResolution();
	halfHeight_ = tan(fieldOfView()*0.5)*dist_;
	halfWidth_  = halfHeight_ * aspect;
}


#endif // CAMERA_H




